// file: $ECE_8993/class/signal/v1.0/sig_exp_0.cc
//

// system include files
//
#include <string.h>

// isip include files
//
#include "signal.h"
#include "signal_constants.h"

#include <integral_constants.h>

// method: expand_cc
//
// arguments:
//  unichar* o_name: (output) name with all environmental symbols
//			     and variables expanded
//  unichar* i_name: (input) name needing expansion
//
// return: a logical value indicating status
//
// this method replaces all environment variables and other such
// things in a string with their equivalent expansions. for example:
// 
//  $HOME/foo.text		=> /u1/person/foo.text
//  foo.text/$HOME/data		=> foo.text//u1/person/data
//
// in other words, variables or aliases anywhere in the string
// will be replaced.
//
// this method now uses popen/pclose rather than complicated
// function calls and string replacements.
//
logical Signal::expand_cc(unichar* o_name_a, unichar* i_name_a) {

  // check for bad filenames
  //
  if ((o_name_a == (unichar*)NULL) || (i_name_a == (unichar*)NULL)) {
    return ISIP_FALSE;
  }
  else {
    
    // scan the incoming string for special characters
    //
    long len = strlen((char*)i_name_a);
    logical flag_match = ISIP_FALSE;
    
    for (long i = 0; i < len; i++) {
      if (strchr(SIGNAL_ENV_DELIMITERS, i_name_a[i]) != (char*)NULL) {
	flag_match = ISIP_TRUE;
	break;
      }
    }
    
    // if necessary, expand the string
    //
    if (flag_match == ISIP_TRUE) {
      
      // build a Unix command
      //
      unichar buf[ISIP_MAX_FNAME_SIZE];
      sprintf((char*)buf, "echo %s\n", (char*)i_name_a);
      
      // execute the pipe
      //
      FILE* fp = popen((char*)buf, "r");
      fgets((char*)o_name_a, ISIP_MAX_FNAME_SIZE, fp);
      pclose(fp);
      
      // strip the trailing linefeed
      //
      len = strlen((char*)o_name_a) - (long)1;
      o_name_a[len] = ISIP_NULL;
      
      // make sure everything got expanded
      //
      flag_match = ISIP_FALSE;
      for (long i=0; i<len; i++) {
	if (strchr(SIGNAL_ENV_DELIMITERS, o_name_a[i]) != (char*)NULL) {
	  return ISIP_FALSE;
	}
      }
    }
    
    // else, just copy the string
    //
    else {
      strcpy((char*)o_name_a, (char*)i_name_a);
    }
    
    // exit gracefully
    //
    return ISIP_TRUE;
    
  }
}




