#! /usr/local/bin/wish -f

# file: $ISIP_DISPLAY_SIGNAL/gui/src/bindings.tcl
#
# definitions of principal key bindings and on-line functional help 
# for the DISPLAY_SIGNAL
#

# declare globals
#
global p wrn

global play_mode
global current_channel
global loadflag

# allow for automatic resizing of the plots
#
bind $p.chan0_plot <Configure> { update_window }
bind $p.energy0_plot <Configure> { update_window }
bind $p.spec0_plot <Configure> { update_window }

# bind the graphics menu to the right mouse button
#
bind $p <ButtonPress-3> {tk_popup $p.menu_graphics %X %Y}

bind $p <Alt-Up> {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }

    $p config -cursor watch
    update
    zoom_out_proc
    $p config -cursor top_left_arrow
}

bind $p <Alt-Down> {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }

    $p config -cursor watch
    update
    zoom_in_proc
    $p config -cursor top_left_arrow
}

bind $p <Alt-c> {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }

    config_proc
    update
}

bind $p <Alt-h> {
    help_proc
}

bind $p <Alt-l> {

    $p config -cursor watch
    update
    load_cfgfile_proc
    $p config -cursor top_left_arrow
}

bind $p <Alt-m> {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }

    play_brackets_proc
}

bind $p <Alt-q> {
    quit_proc
}

bind $p <Alt-b> {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }

    $p config -cursor watch
    zoom_brackets_proc
    $p config -cursor top_left_arrow
}

bind $p <Alt-u> {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }

    play_utt_proc 0
}

bind $p <Alt-v> {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }

    verify_proc
    update
}

bind $p <Alt-w> {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }

    play_window_proc
}

bind $p <Alt-n> {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }

    $p config -cursor watch
    next_utt_proc
    $p config -cursor top_left_arrow
}

bind $p <Alt-p> {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }

    $p config -cursor watch
    prev_utt_proc
    $p config -cursor top_left_arrow
}

bind $p <Alt-z> {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }

    $p config -cursor watch
    update
    zoom_full_proc
    $p config -cursor top_left_arrow
}

bind $p <Alt-Right> {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }

    $p config -cursor watch
    update
    window_ahead_proc
    $p config -cursor top_left_arrow
}

bind $p <Alt-Left> {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }

    $p config -cursor watch
    update
    window_back_proc
    $p config -cursor top_left_arrow
}

bind $p.chan0_plot <space> {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }

    # play between brackets
    #
    play_brackets_proc
}

bind $p.energy0_plot <space> {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }

    # play between brackets
    #
    play_brackets_proc
}

bind $p.spec0_plot <space> {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }

    # play between brackets
    #
    play_brackets_proc
}

# focus settings for the signal plot widget canvas
#
bind $p.chan0_plot <Enter> {
    focus %W
}

# focus settings for the energy plot widget canvas
#
bind $p.energy0_plot <Enter> {
    focus %W
}

# focus settings for the energy plot widget canvas
#
bind $p.spec0_plot <Enter> {
    focus %W
}

# mouse bindings for the signal plot area
#
bind $p.chan0_plot <ButtonPress-1> {

    get_start_coords %x %y
}
bind $p.chan0_plot <ButtonRelease-1> {

    get_end_coords %x %y
}

# mouse bindings for the energy plot area
#
bind $p.energy0_plot <ButtonPress-1> {

    get_start_coords %x %y
}
bind $p.energy0_plot <ButtonRelease-1> {
    
    get_end_coords %x %y
}

# mouse bindings for the spectrogram area
#
bind $p.spec0_plot <ButtonPress-1> {

    get_start_coords %x %y
}
bind $p.spec0_plot <ButtonRelease-1> {

    get_end_coords %x %y
}

# mouse motion for the signal plot
#
bind $p.chan0_plot <B1-Motion> {

    bracket_draw_proc %x %y
}
bind $p.chan0_plot <Motion> {

    get_time_proc %x %y
}

# mouse motion for the energy plot
#
bind $p.energy0_plot <B1-Motion> {

    bracket_draw_proc %x %y
}
bind $p.energy0_plot <Motion> {

    get_time_proc %x %y
}

# mouse motion for the energy plot
#
bind $p.spec0_plot <B1-Motion> {

    bracket_draw_proc %x %y
}
bind $p.spec0_plot <Motion> {

    get_time_proc %x %y
} 

#--------------------------------------------------------------------------
# bind the middle mouse to help for each button
#--------------------------------------------------------------------------

# main control buttons
#
set load_hlpstr "Load\n\nLoad the desired configuration file. The \
	configuration file contains the parameters required to load \
	the speech and transcription data."
bind $p.load <ButtonPress-2> {
    help_item_proc $load_hlpstr
}

set config_hlpstr "Config\n\nSet or change the desired configuration \
	options. These allow you to load specific audio and \
	transcription files, set the audio play environment and other \
	useful system parameters."
bind $p.config <ButtonPress-2> {
    help_item_proc $config_hlpstr
}

set help_hlpstr "Help\n\nView the on-line documents containing a list \
	of available key-bindings, the recommended transcription \
	guidelines and some general instructions regarding how to \
	transcribe data using the DISPLAY_SIGNAL."
bind $p.help <ButtonPress-2> {
    help_item_proc $help_hlpstr
}

set quit_hlpstr "Quit\n\nClose the DISPLAY_SIGNAL and exit. The DISPLAY_SIGNAL will \
	prompt you to save data if necessary."
bind $p.quit <ButtonPress-2> {
    help_item_proc $quit_hlpstr
}

# volume and amplitude gain
#
set volume_label_hlpstr "Volume Match\n\nSet the volume of both channels \
	to the larger of the two slider values."
bind $p.volume_label <ButtonPress-2> {
    help_item_proc $volume_label_hlpstr
}

set gain_label_hlpstr "Amplitude Gain\n\nChange the amplitude of the \
	waveforms on display."
bind $p.gain_label <ButtonPress-2> {
    help_item_proc $gain_label_hlpstr
}

set contrast_label_hlpstr "Contrast Gain\n\nChange the contrast of the \
	spectrogram on display."
bind $p.con_label <ButtonPress-2> {
    help_item_proc $contrast_label_hlpstr
}

set brightness_label_hlpstr "Brightness Gain\n\nChange the brightness of the \
	spectrogram on display."
bind $p.brt_label <ButtonPress-2> {
    help_item_proc $brightness_label_hlpstr
}

# quick play
#
set qplay_hlpstr "Quick Play\n\nPlay the current utterance (0), or any \
	of the two previous (-1, -2) or two next (+1, +2) utterances \
	adjacent to the current utterance.\n\nSee help on \"Play\" by \
	clicking the middle mouse button on the \"Play\" button for \
	details on the audio play utility."
bind $p.qplay_label <ButtonPress-2> {
    help_item_proc $qplay_hlpstr
}
bind $p.play_2b <ButtonPress-2> {
    help_item_proc $qplay_hlpstr
}
bind $p.play_1b <ButtonPress-2> {
    help_item_proc $qplay_hlpstr
}
bind $p.play_0 <ButtonPress-2> {
    help_item_proc $qplay_hlpstr
}
bind $p.play_1a <ButtonPress-2> {
    help_item_proc $qplay_hlpstr
}
bind $p.play_1b <ButtonPress-2> {
    help_item_proc $qplay_hlpstr
}

# utterance traversal and control
#
set clear_hlpstr "Clear\n\nClear the list box of all audio file that have \
	been selected in the listbox. You can select the audio files by \
	dragging the mouse over them within the confines of the listbox."
bind $p.clear <ButtonPress-2> {
    help_item_proc $clear_hlpstr
}

set play_hlpstr "Play Mode\n\nPlay all selected audio files in the listbox. \
	You can select the audio files by dragging the mouse over them \
	within the confines of the listbox. The play function can be \
	interrupted and resumed by clicking on the \"Play\" button, which \
	acts as a toggle switch."
bind $p.play <ButtonPress-2> {
    help_item_proc $play_hlpstr
}

set prev_hlpstr "Previous Utterance\n\nSelect the previous utterance \
	and set it as current."
bind $p.prev <ButtonPress-2> {
    help_item_proc $prev_hlpstr
}

set next_hlpstr "Next Utterance\n\nSelect the next utterance \
	and set it as current."
bind $p.next <ButtonPress-2> {
    help_item_proc $next_hlpstr
}
    
#
# end of file
