#! /usr/local/bin/wish -f

# file: $ISIP_DISPLAY_SIGNAL/gui/src/config.tcl 
# 
# procedures to set the configuration parameters for the DISPLAY_SIGNAL
#

# list of procedures:
#
#  load_cfgfile_proc {}
#  update_proc {}
#  update_all_proc {}
#  config_proc {}
#  config_ok_proc {}
#  config_reload_proc {}
#  config_set_proc {}
#

# procedure: load_cfgfile_proc
#
# arguments: none
#
# return: none
#
# procedure to prompt the user to load data
#
proc load_cfgfile_proc {} {

    # declare globals
    #
    global default_data_dir
    global config_file
    global audio_list_file
    global audiolist
    global current_channel
    
    global DISPLAY_SIGNAL_title
    global bg_col
    
    global w
    set w ".cfgfile"
    
    global f
    
    # destroy the call selection window if it exist
    #
    catch {
	
	if {[winfo exists $f] == 1} {
	    destroy $f
	}
    }
    
    # don't recreate the window, try to reuse existing one
    #
    if {[winfo exists $w] == 1} {
        wm deiconify $w
        raise $w
	
    } else {
	
	# set the default values
	#
	set audio_list_file ""
	
	# create a new window
	#
	toplevel $w
        wm title $w "$DISPLAY_SIGNAL_title Load Params"
        wm geometry $w "+200+200"
	wm focusmodel $w active
	
	# make a top and bottom frame
	#
	frame $w.top_f -background $bg_col
	frame $w.bot_f -background $bg_col
	pack $w.top_f -side top -fill both -expand true -padx 1 -pady 1
	pack $w.bot_f -side bottom -fill x -expand true -padx 1 -pady 1
	
	# create buttons to accept / cancel
        #
        button $w.ok -text OK -default active -width 12 -background $bg_col \
		-command {
	    
	    # check if the user has entered a list or a file (RAW)
	    #
	    if {[file extension $cfgfile] == ".list"} {
		
		# set the name of the file containing the list
		#
		set audio_list_file $cfgfile
		
		# set the new config file and close this window
		#
		if {$audio_list_file != ""} {
		    set audio_list_file [expand_env_proc $audio_list_file]
		}

		# check if the config file exists
		#
		if {[file exists $audio_list_file] == 0} {
		    warning_proc "The file you have selected does \
			    not exist please select another file"
		    return
		}
		
		# destroy the load window
		#
		destroy $w

		# read data from the file and set up the utterance list
		#
		load_proc	
	
	    } elseif {[file extension $cfgfile] == ".raw"} {

		# expant the the audio file name
		#
		set audiostr [expand_env_proc $cfgfile]
		
		# add the expanded audio file name
		#
		set templist {}
		lappend templist $audiostr
		set audiolist [concat $templist $audiolist]
		set loadflag 1

		# destroy the load window
		#
		destroy $w

		# reorder the lists in order of start times and
		# set up the current channel as default display
		#
		set_channel_proc $current_channel
		
		# update the display
		#
		update_window

	    } else {
		warning_proc "You have specified an invaild file type"
		return
	    }		
	}
	
	# add the cancel button
	#
	button $w.cancel -text Cancel -command "destroy $w" -width 12 \
		-background $bg_col

	pack $w.ok $w.cancel -side left -in $w.bot_f -fill y -padx 1 -pady 1 \
		-expand true
	
	bind $w <Return> {
 
	    # check if the config file exists
	    #
	    if {[file exists $audio_list_file] == 0} {
		warning_proc "The file you have selected does \
			not exist please enter another file"
		return
	    }

	    # set the new config file and close this window
	    #
	    if {$audio_list_file != ""} {
		set audio_list_file [expand_env_proc $audio_list_file]
	    }
	    destroy $w

	    # read data from the file and set up the utterance list
	    #
	    load_proc
	}
	
	# setup the configuration file labels and entry fields
        #
        label $w.cfgfile_label -text "Audio Parameter:" -anchor w -width 20 \
		-background $bg_col
        entry $w.cfgfile_entry -textvariable cfgfile -width 60 \
		-background $bg_col

	# setup the configuration browse button
        #
        button $w.cfgbrowse -text Browse -background $bg_col -command {
	    set fmask "*"
	    set cfgdir $default_data_dir
	    if {[file exists $config_file] == 1} {
		set cfgdir [file dirname $config_file]
	    }
	    set cfgfile [tk_filesel .filesel -filemask $fmask \
                    -directory $cfgdir]
        }

        pack $w.cfgfile_label $w.cfgfile_entry $w.cfgbrowse -side left \
                -fill both -expand true -padx 1 -pady 1 -in $w.top_f
    }
}

# procedure: config_proc
#
# arguments: none
#
# return: none
#
# this procedure sets up the config pop-up window and processes the
# corresponding inputs
#
proc config_proc {} {

    # set globals
    #
    global audio_file

    global trans0_file

    global audio_server
    global audio_play_device

    global autosave_time
    global pause_time
    global DISPLAY_SIGNAL_id

    global brightness_d
    global contrast_d

    global num_channels
    global display_channel

    global config_flag
    global audio_list_file

    # set local variables
    #
    global transfile
    global callfile

    global lexfile
    global lexicon_file
    global logfile

    global auserver
    global audevice

    global autosave
    global pause
    global transid

    global window_len
    global frame_len
    global preemph_coeff

    global sample_frequency
    global preemph_flag
    global window_option
    global preemphasis_display
    global window_size
    global rms_scale
    global snr
    global snr_flag
   
    global ultra
    global super
    global major

    global reload_flag

    global cfg
    set cfg ".config"

    global DISPLAY_SIGNAL_title

    global bg_col
    global default_data_dir

    # set the config flag to indicate that the procedure has been visited
    #
    set config_flag 1

    # don't recreate the window, try to reuse existing one
    #
    if { [winfo exists $cfg] == 1 } {
        wm deiconify $cfg
        raise $cfg
 
    } else {
 
	# set the default values
	#
	set auserver $audio_server
	set audevice $audio_play_device
	set autosave $autosave_time
	set pause $pause_time
	set transid $DISPLAY_SIGNAL_id
	set snr_flag "Off"
	set snr 0

        # create a new window
        #
        toplevel $cfg
        wm title $cfg "$DISPLAY_SIGNAL_title Configuration"
	wm geometry $cfg "+200+200"
	wm focusmodel $cfg active

	# make a top and bottom frame
	#
	frame $cfg.top1_f -borderwidth 0 -background $bg_col
	frame $cfg.top2_f -borderwidth 0 -background $bg_col
	frame $cfg.top3_f -borderwidth 0 -background $bg_col
	frame $cfg.top4_f -borderwidth 0 -background $bg_col
	frame $cfg.top5_f -borderwidth 0 -background $bg_col
	frame $cfg.foot -borderwidth 1 -background $bg_col
	
	# pack the top and bottom frame
	#
	pack $cfg.top1_f $cfg.top2_f $cfg.top3_f $cfg.top4_f $cfg.top5_f \
		-side top -fill both -expand true 
	pack $cfg.foot -side bottom -fill both -expand true -pady 2
	
	# create buttons to accept / cancel
	#
        button $cfg.ok -text Ok -width 12 -default active \
		-background $bg_col -command {
	    # reset the config flag to indicate that we have left 
            # the config procedure
	    #
	    set config_flag 0
	    config_ok_proc
	}
	button $cfg.cancel -text Cancel -width 12 -background $bg_col \
		-command {
	    # reset the config flag to indicate that we have left 
            # the config procedure
	    #
	    set config_flag 0
	    destroy $cfg
	}

        pack $cfg.ok -side left -in $cfg.foot -fill x -padx 1 -pady 1
        pack $cfg.cancel -side right -in $cfg.foot -fill x -padx 1 -pady 1

	bind $cfg <Return> {
	    # reset the config flag to indicate that we have left 
            # the config procedure
	    #
	    set config_flag 0
	    config_ok_proc
	}
	
	# make frames for audio related parameters entry
	#
	frame $cfg.audio_f -background $bg_col
	frame $cfg.audevice_f -background $bg_col
	frame $cfg.auserver_f -background $bg_col
	frame $cfg.aufile_f -background $bg_col

	# make frames for energy plot parameters entry
	#
	frame $cfg.energy_f -background $bg_col
	frame $cfg.frmlen_f -background $bg_col
	frame $cfg.winlen_f -background $bg_col
	frame $cfg.rms_f -background $bg_col
	frame $cfg.size_f -background $bg_col

	# make frames for energy plot parameters entry
	#
	frame $cfg.spec_f -background $bg_col
	frame $cfg.contrast_f -background $bg_col
	frame $cfg.brightness_f -background $bg_col
	
	# make frames for common attributes
	#
	frame $cfg.snr_f -background $bg_col
	frame $cfg.misc_f -background $bg_col
	frame $cfg.preemp_f -background $bg_col
	frame $cfg.preflg_f -background $bg_col
	frame $cfg.hamflg_f -background $bg_col
	frame $cfg.dispwin_f -background $bg_col
	frame $cfg.freq_f -background $bg_col
	frame $cfg.num_chan_f -background $bg_col
	frame $cfg.disp_chan_f -background $bg_col
	frame $cfg.enable_f -background $bg_col

	pack $cfg.audio_f $cfg.aufile_f $cfg.audevice_f $cfg.auserver_f \
		-side top -fill both -padx 1 -pady 1  -expand true \
		-in $cfg.top2_f

	pack $cfg.energy_f $cfg.frmlen_f $cfg.winlen_f $cfg.rms_f $cfg.size_f \
		-padx 1 -pady 1 -fill both -expand true -in $cfg.top3_f

	pack $cfg.spec_f $cfg.contrast_f $cfg.brightness_f \
		-padx 1 -pady 1 -fill both -expand true -in $cfg.top4_f

	pack $cfg.misc_f $cfg.freq_f $cfg.preemp_f $cfg.snr_f $cfg.preflg_f \
		$cfg.hamflg_f $cfg.dispwin_f $cfg.enable_f $cfg.num_chan_f \
		$cfg.disp_chan_f -padx 1 -pady 1 \
		-fill both -expand true -in $cfg.top5_f

	# pack the lables header for the audio related parameters
	#
	label $cfg.audio_label -text "Audio-related Parameters"  \
		-background $bg_col
	pack $cfg.audio_label -fill both -anchor w -in $cfg.audio_f
    	
	# set up the audio play device
	#
	label $cfg.audevice_label -text "Audio device:" -anchor w -width 20 \
		-background $bg_col
	entry $cfg.audevice_entry -textvariable audevice -width 60 \
		-background $bg_col
	button $cfg.audevsel -text Toggle -background $bg_col -command {
	    if {$audevice == "sparc"} {
		set audevice "dat"
		set auserver ""
	    } elseif {$audevice == "dat"} {
		set audevice "ncd"
		set auserver "$env(DISPLAY)"
	    } elseif {$audevice == "ncd"} {
		set audevice "x86"
		set auserver "speaker"
	    } elseif {$audevice == "x86"} {
		set audevice ""
		set auserver ""
	    } elseif {$audevice == ""} {
		set audevice "sparc"
		set auserver "speaker"
	    }
	}
	pack $cfg.audevice_label $cfg.audevice_entry $cfg.audevsel -side left \
		-fill both -expand true -padx 1 -pady 1 -in $cfg.audevice_f
	
	# setup the audio server
	#
	label $cfg.auserver_label -text "Audio server:" -anchor w -width 20 \
		-background $bg_col
	entry $cfg.auserver_entry -textvariable auserver -width 60 \
		-background $bg_col
	button $cfg.auservsel -text Toggle -background $bg_col -command {
	    if {$audevice == "sparc"} {
		if {$auserver == "speaker"} {
		    set auserver "headphone"
		} elseif {$auserver == "headphone"} {
		    set auserver "line"
		} elseif {$auserver == "line"} {
		    set auserver ""
		} else {
		    set auserver "speaker"
		}
	    } elseif {$audevice == "dat"} {
		if {$auserver != ""} {
		    set auserver ""
		}
	    } elseif {$audevice == "ncd"} {
		if {$auserver == ""} {
		    set auserver "$env(DISPLAY)"
		} else {
		    set auserver ""
		}
	    } elseif {$audevice == "x86"} {
		if {$auserver == "speaker"} {
		    set auserver "headphone"
		} elseif {$auserver == "headphone"} {
		    set auserver "line"
		} elseif {$auserver == "line"} {
		    set auserver ""
		} else {
		    set auserver "speaker"
		}
	    } elseif {$audevice == ""} {
		set auserver ""
	    }
	}
	pack $cfg.auserver_label $cfg.auserver_entry $cfg.auservsel \
		-side left -fill both -expand true -padx 1 -pady 1 \
		-in $cfg.auserver_f
	
	# pack the lables header for the energy plot widget
	#
	label $cfg.energy_label -text "Energy Plot Parameters" \
		-background $bg_col
	pack $cfg.energy_label -fill both -anchor w -in $cfg.energy_f

	# setup the frame length
	#
	label $cfg.frmlen_label -text "Frame Length:" -anchor w -width 20 \
		-background $bg_col
	entry $cfg.frmlen_entry -textvariable frame_len -width 60 \
		-background $bg_col
	label $cfg.foo1 -text "" -width 10 -anchor w -background $bg_col
	pack $cfg.frmlen_label $cfg.frmlen_entry $cfg.foo1 -side left \
		-fill both -expand true -padx 1 -pady 1 -in $cfg.frmlen_f

	# setup the window length
	#
	label $cfg.winlen_label -text "Window Length:" -anchor w -width 20 \
		-background $bg_col
	entry $cfg.winlen_entry -textvariable window_len -width 60 \
		-background $bg_col
	label $cfg.foo2 -text "" -width 10 -anchor w -background $bg_col
	pack $cfg.winlen_label $cfg.winlen_entry $cfg.foo2 -side left \
		-fill both -expand true -padx 1 -pady 1 -in $cfg.winlen_f

	# setup the rms scaling factor
	#
	label $cfg.rms_label -text "Rms Scale:" -anchor w -width 20 \
		-background $bg_col
	entry $cfg.rms_entry -textvariable rms_factor -width 60 \
		-background $bg_col
	label $cfg.foo3 -text "" -width 10 -anchor w -background $bg_col
	pack $cfg.rms_label $cfg.rms_entry $cfg.foo3 -side left \
		-fill both -expand true -padx 1 -pady 1 -in $cfg.rms_f
	
	# set up the energy plot configuration option
	#
	label $cfg.size_label -text "Canvas Size:" -anchor w -width 20 \
		-background $bg_col
	entry $cfg.size_entry -textvariable window_size -width 60 \
		-background $bg_col
	label $cfg.foo4 -text "Percent(%)" -width 10 -anchor w \
		-background $bg_col
	pack $cfg.size_label $cfg.size_entry $cfg.foo4 -side left \
		-fill both -expand true -padx 1 -pady 1 -in $cfg.size_f

	# set up the configuration options for the spectrogram
	#
	label $cfg.spec_label -text "Spectrogram Parameters" \
		-background $bg_col
	pack $cfg.spec_label -fill both -anchor w -in $cfg.spec_f

	# setup the contrast length
	#
	label $cfg.contrast_label -text "Contrast:" -anchor w -width 20 \
		-background $bg_col
	entry $cfg.contrast_entry -textvariable contrast_d -width 60 \
		-background $bg_col
	label $cfg.foo5 -text "" -width 10 -anchor w -background $bg_col
	pack $cfg.contrast_label $cfg.contrast_entry $cfg.foo5 -side left \
		-fill both -expand true -padx 1 -pady 1 -in $cfg.contrast_f

	# setup the brightness length
	#
	label $cfg.brightness_label -text "Brightness:" -anchor w -width 20 \
		-background $bg_col
	entry $cfg.brightness_entry -textvariable brightness_d -width 60 \
		-background $bg_col
	label $cfg.foo6 -text "" -width 10 -anchor w -background $bg_col
	pack $cfg.brightness_label $cfg.brightness_entry $cfg.foo6 \
		-side left -fill both -expand true -padx 1 -pady 1 \
		-in $cfg.brightness_f

	# set up the miscellaneous configuration options
	#
	label $cfg.misc_label -text "Miscellaneous" \
		-background $bg_col
	pack $cfg.misc_label -fill both -anchor w -in $cfg.misc_f
	
	# setup the SNR calculation display facility
	#	
	label $cfg.snr -text "SNR:" -anchor w -width 20 \
		-background $bg_col	
	
	entry $cfg.snrflag -textvariable snr_flag -width 27 \
                -background $bg_col
	
	button $cfg.snrsel -text Toggle -background $bg_col -command {
	    if {$snr_flag == "On"} {
		set snr_flag "Off"
		set snr 0
	    } elseif {$snr_flag == "Off"} {
		set snr_flag "On"
		set snr [snr_proc]
	    }
	}	
	
	entry $cfg.comp -textvariable snr -background $bg_col -width 30
	
	pack $cfg.snr  $cfg.snrflag $cfg.comp $cfg.snrsel -side left \
		-fill both -expand true -padx 1 -pady 1 -in $cfg.snr_f
	
	# set up the window function option
	#
	label $cfg.hamflg_label -text "Window Function:" -anchor w -width 20 \
		-background $bg_col
	entry $cfg.hamflg_entry -textvariable window_option -width 60 \
		-background $bg_col
	button $cfg.hamsel -text Toggle -background $bg_col -command {
	    if {$window_option == "bartlett"} {
		set window_option "blackman"
	    } elseif {$window_option == "blackman"} {
		set window_option "hamming"
	    } elseif {$window_option == "hamming"} {
		set window_option "hanning"
	    } elseif {$window_option == "hanning"} {
		set window_option "rectangular"
	    } elseif {$window_option == "rectangular"} {
		set window_option "bartlett"
	    }
	}
	pack $cfg.hamflg_label $cfg.hamflg_entry $cfg.hamsel -side left \
		-fill both -expand true -padx 1 -pady 1 -in $cfg.hamflg_f

	# define the display window entry
	#
	label $cfg.dispwin_label -text "Display Window:" -anchor w \
		-width 20 -background $bg_col
	entry $cfg.dispwin_entry -textvariable window_maximum -width 60 \
		-background $bg_col
 	label $cfg.foo8 -text "" -width 10 -anchor w -background $bg_col
	pack $cfg.dispwin_label $cfg.dispwin_entry $cfg.foo8 -side left \
		-fill both -expand true -padx 1 -pady 1 -in $cfg.dispwin_f

	# set up the preemphasis flag
	#
	label $cfg.preflg_label -text "Preemphasis:" -anchor w -width 20 \
		-background $bg_col
	entry $cfg.preflg_entry -textvariable preemphasis_display -width 60 \
		-background $bg_col

	button $cfg.presel -text Toggle -background $bg_col -command {
	    if {$preemph_flag == 1} {
		set preemph_flag 0
		set preemphasis_display "Off"
	    } elseif {$preemph_flag == 0} {
		set preemph_flag 1
		set preemphasis_display "On"
	    }
	}
	pack $cfg.preflg_label $cfg.preflg_entry $cfg.presel -side left \
		-fill both -expand true -padx 1 -pady 1 -in $cfg.preflg_f

	# setup the preemphasis coefficient
	#
	label $cfg.preemp_label -text "Preemphasis Coeff:" -anchor w \
		-width 20 -background $bg_col
	entry $cfg.preemp_entry -textvariable preemph_coeff -width 60 \
		-background $bg_col
 	label $cfg.foo9 -text "" -width 10 -anchor w -background $bg_col
	pack $cfg.preemp_label $cfg.preemp_entry $cfg.foo9 -side left \
		-fill both -expand true -padx 1 -pady 1 -in $cfg.preemp_f

	# setup the sample frequency
	#
	label $cfg.freq_label -text "Sample Frequency:" -anchor w \
		-width 20 -background $bg_col
	entry $cfg.freq_entry -textvariable sample_frequency -width 60 \
		-background $bg_col
 	label $cfg.foo10 -text "" -width 10 -anchor w -background $bg_col
	pack $cfg.freq_label $cfg.freq_entry $cfg.foo10 -side left \
		-fill both -expand true -padx 1 -pady 1 -in $cfg.freq_f

	# setup the number of channels
	#
	label $cfg.num_chan_label -text "Channels:" -anchor w \
		-width 20 -background $bg_col
	entry $cfg.num_chan_entry -textvariable num_channels -width 60 \
		-background $bg_col
 	label $cfg.foo11 -text "" -width 10 -anchor w -background $bg_col
	pack $cfg.num_chan_label $cfg.num_chan_entry $cfg.foo11 -side left \
		-fill both -expand true -padx 1 -pady 1 -in $cfg.num_chan_f

	# setup the display channel
	#
	label $cfg.disp_chan_label -text "Display Channel:" -anchor w \
		-width 20 -background $bg_col
	entry $cfg.disp_chan_entry -textvariable disp_chan -width 60 \
		-background $bg_col
 	label $cfg.foo12 -text "" -width 10 -anchor w -background $bg_col
	pack $cfg.disp_chan_label $cfg.disp_chan_entry $cfg.foo12 -side left \
		-fill both -expand true -padx 1 -pady 1 -in $cfg.disp_chan_f

	# setup the spectrogram on/off facility
	#
	label $cfg.enable_label -text "Spectrogram:" -anchor w \
		-width 20 -background $bg_col
	entry $cfg.enable_entry -textvariable spec_enable -width 60 \
		-background $bg_col
	button $cfg.enable -text Toggle -background $bg_col -command {
	    if {$spec_enable == "on"} {
		set spec_enable "off"
		$p.spec0_plot itemconfigure $spec_plot_0 -amp_scale 0
	    } elseif {$spec_enable == "off"} {
		set spec_enable "on"
		$p.spec0_plot itemconfigure $spec_plot_0 \
			-amp_scale $amplitude_gain
	    }
	}
	pack $cfg.enable_label $cfg.enable_entry $cfg.enable -side left \
		-fill both -expand true -padx 1 -pady 1 -in $cfg.enable_f
    }
}

# procedure: config_ok_proc 
#
# arguments: none
#
# return: none
#
# procedure to set entered variables
#
proc config_ok_proc {} {

    # declare globals
    #
    global cfg p

    global reload_flag
    global audio_list_file

    global frame_len
    global window_size
    global signal_ht
    global energy_ht

    global num_channels
    global disp_chan

    # get the height of the frame
    #
    set frame_ht [$p.plot_f cget -height]
	
    # set the new height of the energy canvas
    #
    set energy_ht [expr int((($window_size / 100.00) * $frame_ht))]
	
    # set the new height of the signal frame
    #
    set signal_ht [expr $frame_ht - $energy_ht]
	
    # configure the canvas heights
    #
    $p.energy0_plot configure -height $energy_ht
    $p.chan0_plot configure -height $signal_ht
    
    # error check for the number of channels
    #
    if {$num_channels > 2 || $num_channels < 1} {
	set num_channels 1
    }

    # error check for the display channel
    #
    if {$disp_chan < 0 || $disp_chan > 1} {
	set disp_chan 0
    }

    # error check for the energy plot frame length 
    #
    if {$frame_len <= 0} {
	set frame_len 0.01
    }

    # error check for the window size
    #
    if {$window_size < 0 || $window_size > 100} {
	set window_size 25
    }

    # load the corresponding data if necessary
    #
    if {$reload_flag == 1} {
	
	# check if data needs to be saved before reload
	#
	config_reload_proc
	set reload_flag 0

    } else {

	# set variables to new values and quit
	#
	config_set_proc
	destroy $cfg
    }
}

# procedure: config_reload_proc 
#
# arguments: none
#
# return: none
#
# procedure to confirm reloading of data
#
proc config_reload_proc {} {

    # create globals
    #
    global cfg

    global DISPLAY_SIGNAL_title
    global bg_col

    global trans_list
    global word_list
    global log_list

    global loadflag

    global arraylist
    global keycount
    global keys

    global last_utter
    global last_call

    global callfile_list

    global w_save
    set w_save ".cfg_save"

    global loadflag

    # check if this needs to be done
    #
    if {$loadflag == 0} {
	destroy $cfg
	load_cfgfile_proc
	return
    }

    # create a dialog window
    #
    toplevel $w_save
    wm title $w_save "$DISPLAY_SIGNAL_title Reload"
    wm geometry $w_save "+300+400"
    wm focusmodel $w_save active

    # make a bottom frame
    #
    frame $w_save.foot -background $bg_col
    pack $w_save.foot -side bottom -fill x -pady 2m
    
    button $w_save.save -width 6 -default active -text Yes \
	    -background $bg_col -command {
	
	# set the current fileds
	#
	destroy $w_save
	config_set_proc
	destroy $cfg

	save_proc
	
	# read data from the file and set up the utterance list
	#
	load_cfgfile_proc
    }

    button $w_save.ok -width 6 -text No  -background $bg_col -command {

	# set the current fileds
	#
	destroy $w_save
	config_set_proc
	destroy $cfg

	# read data from the file and set up the utterance list
	#
	load_cfgfile_proc
    }
    button $w_save.cancel -width 6 -text Cancel -background $bg_col -command {
	destroy $w_save
    }
    pack $w_save.save $w_save.ok $w_save.cancel -side left  -fill x \
	    -padx 1m -pady 1m -expand true -in $w_save.foot
    
    # make a message frame
    #
    frame $w_save.top -background $bg_col
    pack $w_save.top -side top -fill x -padx 1m -pady 1m
    
    message $w_save.msg -justify center -width 2i -background $bg_col -text \
	    "This will reload data from file and destroy unsaved work. \
	    Do you want to save data before you reload?"
    pack $w_save.msg -side top -fill both -in $w_save.top

    # bind the return key
    #
    bind $w_save <Return> {
	
	# set the current fileds
	#
	destroy $w_save
	config_set_proc
	destroy $cfg
	
	save_proc

	# read data from the file and set up the utterance list
	#
	load_cfgfile_proc
    }
}

# procedure: config_set_proc
#
# arguments: none
#
# return: none
#
# procedure to set filenames
#
proc config_set_proc {} {

    # declare globals
    #
    global audio_server
    global audio_play_device

    global autosave_time
    global pause_time
    global DISPLAY_SIGNAL_id
    
    global num_channels
    
    # set local variables
    #
    global auserver
    global audevice

    global autosave
    global pause
    global transid
    
    global cutt_start
    global cutt_end
    global maximum_time
    global window_start
    global window_end
    global window_duration
    global window_maximum
    global bracket_start
    global bracket_end
    global num_channels
    global sample_num_bytes
    global sample_frequency
    global audio_file

    global loadflag
    global p

    # check if this needs to be done
    #
    if {$loadflag == 1} {
    
	if {$auserver != ""} {
	    set audio_server [expand_env_proc $auserver]
	}
	if {$audevice != ""} {
	    set audio_play_device [expand_env_proc $audevice]
	}
	if {$autosave != ""} {
	    set autosave_time [expand_env_proc $autosave]
	}
	if {$pause != ""} {
	    set pause_time [expand_env_proc $pause]
	}
	if {$transid != ""} {
	    set DISPLAY_SIGNAL_id [expand_env_proc $transid]
	}

	# make sure that the sample frequency is a real number
	#
	set sample_frequency [eval expr $sample_frequency * 1.0]

	# set the maximum time for the audio file
	#
	set maximum_time [format "%.6f" [snap_sample_proc [eval expr \
		[file size $audio_file] / ($sample_num_bytes * \
		$sample_frequency * $num_channels)]]]

	# set the maximum window size equal to the maximum time if the 
	# maximum time is less that maximum window size
	#
	if {$maximum_time < $window_maximum} {

	    # set the default start and end times
	    #
	    set cutt_start 0
	    set cutt_end $maximum_time
	    set window_start 0
	    set window_end $maximum_time
	    set window_duration $maximum_time
	    set bracket_start 0
	    set bracket_end $maximum_time

	} else {

	    # set the default start and end times
	    #
	    set cutt_start 0
	    set cutt_end $window_maximum
	    set window_start 0
	    set window_end $window_maximum
	    set window_duration $window_maximum
	    set bracket_start 0
	    set bracket_end $window_maximum
	}

	# set the current window display
	#
	curr_window_proc
    }
}

#
# end of file
