#! /usr/local/bin/wish -f

# file: $ISIP_DISPLAY_SIGNAL/gui/src/display.tcl
#
# this file sets up the main segmentation tool widgets
#

# declare global variables
#
global current_channel
global play_mode
global sig_plot_0
global energy_plot_0
global spec_plot_0
global disp_chan

global bracket_signal_start_line
global bracket_signal_end_line
global bracket_energy_start_line
global bracket_energy_end_line
global bracket_spec_start_line
global bracket_spec_end_line

global audio_file
global audio_list

global window_len
global frame_len
global rms_factor
global amplitude_gain

global preemph_coeff
global preemph_flag
global window_option 

global brightness_d
global contrast_d

global loadflag
global DISPLAY_SIGNAL_id

global signal_ht
global energy_ht

global quitflag

global p

# get rid of the old toplevel
#
wm withdraw .
catch {destroy $p}

# set new toplevel
#
toplevel $p
wm title $p "$DISPLAY_SIGNAL_title"

# communicate with the window manager
#
wm protocol $p WM_DELETE_WINDOW {
    quit_proc
    exit
}

wm geometry $p "+0+0"
wm minsize $p 760 430

# create the base frames
#
frame $p.trans_f -borderwidth 1 -background $bg_col -relief flat \
	-height 430 -width 480
frame $p.ctrl_f -borderwidth 1 -background $bg_col -relief ridge \
	-height 430 -width 280

pack $p.trans_f -side left -fill both -expand true
pack $p.ctrl_f -side right -fill both

# create subframes for transcription area
#
frame $p.top_f -borderwidth 1 -relief ridge -height 430 -width 480 \
	-background $bg_col
pack $p.top_f -side top -fill both -expand true -in $p.trans_f

# create logo subframe
#
frame $p.logo_f -borderwidth 1 -relief flat -height 40 -width 280 \
	-background $bg_col
pack $p.logo_f -padx 1 -pady 1 -side top -fill both -in $p.ctrl_f

# create subframes for tool control panel
#
frame $p.ctr1_f -borderwidth 0 -relief flat -height 40 -width 280 \
	-background $bg_col
frame $p.ctr2_f -borderwidth 0 -relief flat -height 40 -width 280 \
	-background $bg_col
frame $p.ctr3_0_f -borderwidth 0 -relief flat -height 40 -width 280 \
	-background $bg_col
frame $p.ctr3_1_f -borderwidth 0 -relief flat -height 40 -width 280 \
	-background $bg_col
frame $p.ctr3_2_f -borderwidth 0 -relief flat -height 40 -width 280 \
	-background $bg_col
frame $p.ctr3_3_f -borderwidth 0 -relief flat -height 40 -width 280 \
	-background $bg_col
frame $p.ctr4_f -borderwidth 0 -relief flat -height 40 -width 280 \
	-background $bg_col
frame $p.ctr5_f -borderwidth 0 -relief flat -height 40 -width 280 \
	-background $bg_col
frame $p.ctr6_f -borderwidth 0 -relief flat -height 40 -width 280 \
	-background $bg_col

pack $p.ctr1_f $p.ctr2_f $p.ctr5_f $p.ctr6_f $p.ctr3_0_f \
	$p.ctr3_1_f $p.ctr3_2_f $p.ctr3_3_f $p.ctr4_f -padx 1 -pady 1 \
	-side top -fill both -in $p.ctrl_f

# creates subframe for utterance control panel
#
frame $p.utt_f -borderwidth 1 -relief flat -height 70 -width 280 \
	-background $bg_col
frame $p.utr_f -borderwidth 0 -relief flat -height 70 -width 280 \
	-background $bg_col
frame $p.utr1_f -borderwidth 0 -relief flat -height 50 -width 280 \
	-background $bg_col
frame $p.utr2_f -borderwidth 0 -relief flat -height 20 -width 280 \
	-background $bg_col

pack $p.utr1_f -padx 1 -side top -fill both -expand true -in $p.utr_f
pack $p.utr2_f -padx 1 -side top -fill both -in $p.utr_f
pack $p.utr_f -padx 1 -pady 1 -side right -fill both -expand true -in $p.utt_f
pack $p.utt_f -padx 1 -pady 1 -side top -fill both -expand true -in $p.ctrl_f

# create subframes for graphics area
#
frame $p.time_f -borderwidth 1 -relief flat -height 40 -width 480 \
	-background $bg_col
frame $p.plot_f -borderwidth 0 -relief flat -height 260 -width 480 \
	-background $bg_col
frame $p.spec_f -borderwidth 0 -relief flat -height 130 -width 480 \
	-background $bg_col

pack $p.time_f -fill both -expand true -in $p.top_f
pack $p.plot_f -fill both -expand true -in $p.top_f
pack $p.spec_f -fill both -expand true -in $p.top_f

# add the buttons for control panel
#
button $p.load -text Load -width 6 -background $bg_col -command {

    # set the file mask
    #
    set fmask "*"

    # set a default directory value
    #
    set cfgdir $default_data_dir

    if {[file exists $config_file] == 1} {
		set cfgdir [file dirname $config_file]
    }
    set cfgfile [tk_filesel .filesel -filemask $fmask \
	    -directory $cfgdir]

    # check if the user hit the Cancel key instead of the quit key
    #
    if {$quitflag == 1} {
	return
    }

    # check if the user has entered a list or a file (RAW)
    #
    if {[file extension $cfgfile] == ".list"} {

	# remember the new directory path
	#
	set default_data_dir [file dirname $cfgfile]
	
	# set the name of the file containing the list
	#
	set audio_list_file $cfgfile
	
	# set the new config file and close this window
	#
	if {$audio_list_file != ""} {
	    set audio_list_file [expand_env_proc $audio_list_file]
	}
	
	# check if the config file exists
	#
	if {[file exists $audio_list_file] == 0} {
	    warning_proc "The file you have selected does \
		    not exist please select another file"
	    return
	}
	
	# read data from the file and set up the utterance list
	#
	load_proc	
	
    } elseif {[file extension $cfgfile] == ".raw"} {
	
	# remember the new directory path
	#
	set default_data_dir [file dirname $cfgfile]

	# expant the the audio file name
	#
	set audiostr [expand_env_proc $cfgfile]
	
	# add the expanded audio file name
	#
	set templist {}
	lappend templist $audiostr
	set audiolist [concat $templist $audiolist]
	set loadflag 1
	
	# reorder the lists in order of start times and
	# set up the current channel as default display
	#
	set_channel_proc $current_channel
	
	# update the display
	#
	update_window
      
    } else {
	warning_proc "You have specified an invaild file type"
	return
    }		
}
button $p.config -text Config -width 6 -background $bg_col -command {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }

    config_proc
}
button $p.help -text Help -width 6 -background $bg_col -command {
    help_proc
}
button $p.quit -text Quit -width 6 -background $bg_col -command {
    quit_proc
}
button $p.clear -text "Clear" -width 6 -background $bg_col \
	-command {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }

    clear_list_proc
}
button $p.play -text "Play" -width 6 -background $bg_col \
	-command {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }

    verify_proc
    update
}
button $p.ahead -text "Window Ahead" -width 6 -background $bg_col \
	-command {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }

    window_ahead_proc
}
button $p.back -text "Window Back" -width 6 -background $bg_col \
	-command {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }

    window_back_proc
}

pack $p.load $p.config -side left -padx 1 -pady 2 -fill both \
	-expand true -in $p.ctr1_f
pack $p.help $p.quit -side left -padx 1 -pady 2 -fill both \
	-expand true -in $p.ctr2_f
pack $p.clear $p.play -side left -padx 1 -pady 2 -fill both \
	-expand true -in $p.ctr5_f
pack $p.ahead $p.back -side left -padx 1 -pady 2 -fill both \
	-expand true -in $p.ctr6_f

# add the volume control
#
label $p.volume_label -justify left -text "Volume:" -width 11 \
	-anchor w -background $bg_col    
scale $p.volume0 -variable volume_ch0 -from 0 -to 5 -digits 0 \
	-resolution 0.5 -showvalue false -orient horiz -width 10 \
	-sliderlength 20 -background $bg_col -command { volume_proc 0 }
pack $p.volume_label -side left -fill both -in $p.ctr3_0_f
pack $p.volume0 -side left -fill both -expand true -in $p.ctr3_0_f

# add the amplitude scale factor
#
label $p.gain_label -background $bg_col -anchor w -text "Amplitude:" -width 11
scale $p.gain_scale -variable amplitude_gain -from 0 -to 5 -digits 0 \
	-resolution 0.5 -showvalue false -orient horiz -width 10 \
	-sliderlength 20 -background $bg_col -command { update_gain_proc }
pack $p.gain_label -side left -fill both -in $p.ctr3_1_f
pack $p.gain_scale -side left -fill both -expand true -in $p.ctr3_1_f

# add the brightness scale factor
#
label $p.brt_label -background $bg_col -anchor w -text "Brightness:" -width 11
scale $p.brt_scale -variable brightness_d -from 0 -to 5000 -digits 0 \
	-bigincrement 0 -resolution 100.0 -showvalue false -orient horiz \
	-width 10 -sliderlength 20 -background $bg_col \
	-command { update_brightness_proc }
pack $p.brt_label -side left -fill both -in $p.ctr3_2_f
pack $p.brt_scale -side left -fill both -expand true -in $p.ctr3_2_f

# add the contrast scale factor
#
label $p.con_label -background $bg_col -anchor w -text "Contrast:" -width 11
scale $p.con_scale -variable contrast_d -from 0 -to 5 -digits 0 \
	-resolution 0.5 -showvalue false -orient horiz -width 10 \
	-sliderlength 20 -background $bg_col -command {update_contrast_proc}
pack $p.con_label -side left -fill both -in $p.ctr3_3_f
pack $p.con_scale -side left -fill both -expand true -in $p.ctr3_3_f

# add the utterance play buttons
#
label $p.qplay_label -background $bg_col -anchor w -text "QuickPlay:" -width 11
button $p.play_2b -text -2 -width 1 -command {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }
    play_utt_proc -2} -background $bg_col
button $p.play_1b -text -1 -width 1 -command {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }
    play_utt_proc -1} -background $bg_col
button $p.play_0 -text 0 -width 1 -command {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }
    play_utt_proc 0} -background $bg_col
button $p.play_1a -text +1 -width 1 -command {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }
    play_utt_proc +1} -background $bg_col
button $p.play_2a -text +2 -width 1 -command {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }
    play_utt_proc +2} -background $bg_col
pack $p.qplay_label -side left -fill both -expand true -in $p.ctr4_f
pack $p.play_2b $p.play_1b $p.play_0 $p.play_1a $p.play_2a -side left \
	-fill both -expand true -in $p.ctr4_f

# add the listbox stuff
#
listbox $p.list -selectmode extended -yscrollcommand "$p.yscroll set"\
	-width 12 -background $bg_col 
scrollbar $p.yscroll -command "$p.list yview" -background $bg_col 
pack $p.list -side left -padx 1 -pady 1 -fill both -expand true -in $p.utr1_f
pack $p.yscroll -side right -padx 1 -pady 1 -fill y -in $p.utr1_f

# key bindings for the listbox
#
bind $p.list <Double-Button-1> {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }

    $p config -cursor watch
    set index [lindex [$p.list curselection] 0]
    set_current_proc $index
    $p config -cursor top_left_arrow
}
bind $p.list <Enter> {

    # return if nothing is loaded
    #
    catch {
	if {$loadflag == 0} {
	    return
	}
	
	$p.list selection clear 0 end
	$p.list selection set $curr_ind
    }
}

# add the prev/next utterance buttons
#
button $p.prev -bitmap "@$prev_button" -background $bg_col -command {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }

    $p config -cursor watch
    prev_utt_proc
    $p config -cursor top_left_arrow
}
button $p.next -bitmap "@$next_button" -background $bg_col -command {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }

    $p config -cursor watch
    next_utt_proc
    $p config -cursor top_left_arrow
}
pack $p.prev -side left -padx 1 -pady 1 -fill both -in $p.utr2_f
pack $p.next -side right -padx 1 -pady 1 -fill both -in $p.utr2_f

# add the logo stuff
#
label $p.title -text "$DISPLAY_SIGNAL_title" -background $bg_col -width 10
image create photo logo -file "$isip_logo" -format gif -palette 255/255/255
label $p.logo -image logo -background $bg_col
pack $p.title $p.logo -side left -padx 1 -fill x -expand true -in $p.logo_f

# create the graphics pop-up menu
#
menu $p.menu_graphics -relief ridge -title "Graphics Options" \
	-background $bg_col
$p.menu_graphics add command -label "Audio Play" \
	-background $bg_col -font $font_3 -state disabled -command {}
$p.menu_graphics add command -label "Play between marks" \
	-command {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }

    set play_mode 0
    play_brackets_proc
    set play_mode $current_channel
}
$p.menu_graphics add command -label "Play window contents" \
	-command {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }

    set play_mode 0
    play_window_proc
    set play_mode $current_channel
}
$p.menu_graphics add separator
$p.menu_graphics add command -label "Navigation" -background $bg_col \
	-font $font_3 -state disabled -command {}
$p.menu_graphics add command -label "Window Ahead" -accelerator "Alt+Right" \
	-command {
    $p config -cursor watch
    window_ahead_proc
    $p config -cursor top_left_arrow
}
$p.menu_graphics add command -label "Window Back" -accelerator "Alt+Left" \
	-command {
    $p config -cursor watch
    window_back_proc
    $p config -cursor top_left_arrow
}
$p.menu_graphics add separator
$p.menu_graphics add command -label "Zoom" -background $bg_col \
	-font $font_3 -state disabled -command {}
$p.menu_graphics add command -label "Zoom brackets" -accelerator "Alt+b" \
	-command {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }

    zoom_brackets_proc 
}
$p.menu_graphics add command -label "Zoom in" -accelerator "Alt+Down" \
	-command {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }

    $p config -cursor watch
    zoom_in_proc
    $p config -cursor top_left_arrow
}
$p.menu_graphics add command -label "Zoom out" -accelerator "Alt+Up" \
	-command {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }

    $p config -cursor watch
    zoom_out_proc
    $p config -cursor top_left_arrow
}
$p.menu_graphics add command -label "Zoom full" -accelerator "Alt+z" \
	-command {

    # return if nothing is loaded
    #
    if {$loadflag == 0} {
	return
    }

    $p config -cursor watch
    zoom_full_proc
    $p config -cursor top_left_arrow
}
$p.menu_graphics add separator
$p.menu_graphics add command -label "Misc" -background $bg_col \
	-font $font_3 -state disabled -command {}
$p.menu_graphics add command -label "Quit" -accelerator "Alt+q" \
	-command "quit_proc"

# set up the canvas for the wave plot
#
canvas $p.energy0_plot -relief flat -height 65 -width 480 -borderwidth 1 \
	-background $bg_col
pack $p.energy0_plot -fill both -padx 2 -pady 2 -in $p.plot_f

canvas $p.chan0_plot -relief flat -height 195 -width 480 -borderwidth 1 \
	-background $bg_col
pack $p.chan0_plot -fill both -padx 2 -pady 2 -in $p.plot_f

canvas $p.spec0_plot -relief flat -height 130 -width 480 -borderwidth 1 \
	-background $bg_col
pack $p.spec0_plot -fill both -padx 2 -pady 2 -in $p.spec_f

# set up time labels
#
label $p.dur_l -justify left -anchor w -text "W: \[$wstart , $wend\] \
	B: \[$bstart , $bend :: $bdur\] T: \[$time\]" \
	-width 88 -background $bg_col
pack $p.dur_l -side left -fill both -in $p.time_f

# set the energy plot widget
#
set energy_plot_0 [$p.energy0_plot create energy_plot 0 0 \
	-num_chan $num_channels -amp_scale $amplitude_gain \
	-num_bytes $sample_num_bytes -swap_bytes $byte_swap \
	-display_chan $disp_chan -use_preemph $preemph_flag \
	-sample_freq $sample_frequency -start_time 0.0 -winlen $window_len \
	-filter $window_option -rms_scale $rms_factor \
	-end_time 0.0 -framelen $frame_len -preemph_coeff $preemph_coeff \
	-borderwidth 0 -relief flat]

# set the signal plot widget
#
set sig_plot_0 [$p.chan0_plot create signal_plot 0 0 \
	-num_chans $num_channels -num_bytes $sample_num_bytes \
	-swap_bytes $byte_swap -display_chan $disp_chan \
	-sample_freq $sample_frequency -start_time 0.0 \
	-end_time 0.0 -gain_factor $amplitude_gain \
	-borderwidth 0 -relief flat]

# set the spectrogram plot widget
#
set spec_plot_0 [$p.spec0_plot create spectrogram_plot 0 0 \
	-num_chan $num_channels  -contrast $contrast_d \
	-amp_scale $amplitude_gain -fft_len $fftlen_d \
	-num_bytes $sample_num_bytes -brightness $brightness_d \
	-swap_bytes $byte_swap -display_chan $disp_chan \
	-use_preemph $preemph_flag \
	-sample_freq $sample_frequency -start_time 0.0 -winlen $window_len \
	-filter $window_option -end_time 0.0 -framelen $frame_len \
	-preemph_coeff $preemph_coeff -borderwidth 0 -relief flat]

# set the initial bracket lines
#
set bracket_signal_start_line [$p.chan0_plot create line 0 0 0 \
	$signal_ht -width 1.0]
set bracket_signal_end_line [$p.chan0_plot create line 0 0 0 \
	$signal_ht -width 1.0]

set bracket_energy_start_line [$p.energy0_plot create line 0 0 0 \
	$energy_ht -width 1.0]
set bracket_energy_end_line [$p.energy0_plot create line 0 0 0 \
	$energy_ht -width 1.0]

set bracket_spec_start_line [$p.spec0_plot create line 0 0 0 \
	[winfo height $p.spec0_plot] -width 1.0]
set bracket_spec_end_line [$p.spec0_plot create line 0 0 0 \
	[winfo height $p.spec0_plot] -width 1.0]

#
# end of file
