#! /usr/local/bin/wish -f

# file: $ISIP_DISPLAY_SIGNAL/gui/src/load.tcl
#
# procedures for initially setting up the DISPLAY_SIGNAL by reading in the 
# transcription and word alignment files
#

# list of procedures:
#
#  load_proc {}
#  read_file_proc {text_area filename}
#

# procedure: load_proc
#
# arguments: none
#
# return: none
#
# procedure to read data from file and set up utterance lists 
#
proc load_proc {} {

    # declare globals
    #
    global loadflag
    global current_channel

    global audiolist
    global audio_list_file

    global p 

    # read in the audio file list
    #
    # make sure the audiofile exists and is readable
    #
    if {[file readable $audio_list_file] != 1} {
	warning_proc "Cannot open $audio_list_file. Please make sure the file \
		exists and has read permissions."
	return
    } else {
	
	# open the audio file list
	#
	set audioptr [open $audio_list_file r]
	set loadflag 1
    }

    # create a temporary audio list
    #
    while {![eof $audioptr]} {
	
	# read in the file line by line
	#
	set value [gets $audioptr]
	set audiostr [expand_env_proc $value]

	# ignore all blank lines
	#
	if {[string match "" $audiostr] == 0} {
			
	    # add the parsed line to the audiolist
	    #
	    lappend audiolist $audiostr
	}
    }

    # close the file
    #
    close $audioptr

    # reorder the lists in order of start times and
    # set up the current channel as default display
    #
    set_channel_proc $current_channel
    
    # update the display
    #
    update_window
}

# procedure: read_file_proc
#
# arguments:
#  text_area:  the text area on the DISPLAY_SIGNAL   to be manipulated  
#  filename: file to read data from
#
# return: none
#
# procedure to read given filename into given text area
#
proc read_file_proc {text_area filename} {

    # enable text
    #
    $text_area configure -state normal
    $text_area delete 1.0 end
    
    # make sure the config file exists and is readable
    #
    if {[file readable $filename] != 1} {
	warning_proc "Cannot open the file $filename. Please \
		make sure the file exists and has read permissions."
	return
    }

    # open file
    #
    set fileptr [open $filename r]
    while {![eof $fileptr]} {
	
	# read the file line by line
	#
	set str [gets $fileptr]
	
	# display it in the help window
	#
	$text_area insert end "$str\n"
    }
    
    # disable text
    #
    $text_area configure -state disabled
    
    # close file
    #
    close $fileptr
}

#
# end of file
