#! /usr/local/bin/wish -f

# file: $ISIP_DISPLAY_SIGNAL/gui/src/main.tcl
#
# main source program for the isip speech display system
#

# set up the home path
#
global ISIP_DISPLAY_SIGNAL_HOME
set ISIP_DISPLAY_SIGNAL_HOME "$env(ISIP_DISPLAY_SIGNAL)/gui/src"

# source global variables
#
source $ISIP_DISPLAY_SIGNAL_HOME/globals.tcl

# set up the display and principal key bindings
#
source $ISIP_DISPLAY_SIGNAL_HOME/display.tcl
source $ISIP_DISPLAY_SIGNAL_HOME/bindings.tcl

# the configuration options window
#
source $ISIP_DISPLAY_SIGNAL_HOME/config.tcl
source $ISIP_DISPLAY_SIGNAL_HOME/env.tcl
source $ISIP_DISPLAY_SIGNAL_HOME/help.tcl

# commands pertaining to audio play
#
source $ISIP_DISPLAY_SIGNAL_HOME/audio.tcl

# updating the window display
#
source $ISIP_DISPLAY_SIGNAL_HOME/window.tcl
source $ISIP_DISPLAY_SIGNAL_HOME/brackets.tcl
source $ISIP_DISPLAY_SIGNAL_HOME/coords.tcl

# plotting and zooming
#
source $ISIP_DISPLAY_SIGNAL_HOME/zoom.tcl
source $ISIP_DISPLAY_SIGNAL_HOME/plot.tcl
source $ISIP_DISPLAY_SIGNAL_HOME/channel.tcl

# set up the list box
#
source $ISIP_DISPLAY_SIGNAL_HOME/list.tcl
source $ISIP_DISPLAY_SIGNAL_HOME/load.tcl
source $ISIP_DISPLAY_SIGNAL_HOME/utter.tcl

# the verification mode
#
source $ISIP_DISPLAY_SIGNAL_HOME/verify.tcl

# file selection menu 
#
source $ISIP_DISPLAY_SIGNAL_HOME/filesel.tcl

# shut down and exit
#
source $ISIP_DISPLAY_SIGNAL_HOME/quit.tcl

# SNR computation 
#
source $ISIP_DISPLAY_SIGNAL_HOME/snr.tcl

# initialize current time
#
global time_updated
set time_updated [clock seconds]

# start with updated windows
#
update

# read the command line arguments if any exists
#
if {$argc > 0} {
 
    # check if the command line file is a list or audio (RAW) files
    #
    if {($argc == 1) && ([file extension [lindex $argv 0]] == ".list")} {
	
	# set the name of the file containing the list
	#
	set audio_list_file [lindex $argv 0]
	
	# check if the config file exists
	#
	if {[file exists $audio_list_file] == 0} {

	    warning_proc "The file you have selected does \
		    not exist please select another file"
	    return
	}

	# read data from the file and set up the utterance list
	#
	load_proc	

    } else {
	
	# set the audiolist 
	#
	for {set i 0} {$i < $argc } {incr i} {
            
	    # check the file type to make sure that it is supported
	    #
	    if {[file extension [lindex $argv $i]] == ".raw"} {

		# check if the config file exists
		#
		if {[file exists [lindex $argv $i]] == 0} {
		    warning_proc "The file you have selected does \
			    not exist please select another file"
		} else {
		    
		    # append the file to the audio list
		    #
		    lappend audiolist [lindex $argv $i]
		}

	    } else {

		warning_proc "The file [lindex $argv $i] is not valid. You \
			have either specified an invalid file (make sure \
			that it is an audio file of type RAW) or you may have \
			not specified a list of audio file by itself."
	    }

	    # set the load flag
	    #
	    set loadflag 1

	    # reorder the lists in order of start times and
	    # set up the current channel as default display
	    #
	    set_channel_proc $current_channel
	    
	    # update the display
	    #
	    update_window
	}
    }
}

#
# end of file
