#! /usr/local/bin/wish -f

# file: $ISIP_DISPLAY_SIGNAL/gui/src/plot.tcl
#
# procedures for plotting data
#

# list of procedures:
#
#  plot_channel_proc {}
#  plot_turns_proc {uttlist ch y_in}
#  update_gain_proc {gain}
#  update_brightness_proc {gain}
#  update_contrast_proc {gain}
#  tk_popup {menu x y {entry {}}}
#

# procedure: plot_channel_proc
#
# arguments: none
#
# return: none
#
# procedure to plot the signal for the current channel
#
proc plot_channel_proc {} {
    
    # declare globals
    #
    global window_start
    global window_end
    global window_duration
    global maximum_time

    global bracket_start
    global bracket_end
    
    global bracket_signal_start_line
    global bracket_signal_end_line
    global bracket_energy_start_line
    global bracket_energy_end_line
    global bracket_spec_start_line
    global bracket_spec_end_line

    global sig_plot_0
    global spec_plot_0
    global energy_plot_0

    global signal_ht
    global spec_ht
    global energy_ht

    global chan0_uttlist

    global tscale_sep
    global tscale_ht_ultra
    global tscale_ht_super
    global tscale_ht_major
    global tscale_ht_minor
    global tscale_frame

    global window_len
    global frame_len
    global preemph_coeff
    global preemph_flag
    global window_option
    global window_size
    global rms_factor
    global amplitude_gain
    global sample_frequency
    global plot_width

    global fftlen_d
    global brightness_d
    global contrast_d
    global spec_enable
    global audio_file
    global flag_minor
    global font_3
    global ultra
    global super
    global major

    global num_channels
    global disp_chan

    global p

    # update plot width and window start and end times
    #
    set plot_width [winfo width $p.energy0_plot]

    # make sure this is a valid plot
    #
    if {$window_duration <= 0} {
	warning_proc "Error in plotting --- signal duration is 0 or \
		smaller. \n\nPlease check and adjust time marks \
		appropriately."
	return
    }
 
    # plot the energy of the signal
    #
    $p.energy0_plot itemconfigure $energy_plot_0 -start_time $window_start \
	    -end_time $window_end -height $energy_ht -width $plot_width \
	    -use_preemph $preemph_flag -filter $window_option \
	    -framelen $frame_len -preemph_coeff $preemph_coeff \
	    -rms_scale $rms_factor -amp_scale $amplitude_gain \
	    -sample_freq $sample_frequency -file $audio_file \
	    -num_chan $num_channels -display_chan $disp_chan \
	    -winlen $window_len -borderwidth 0

    # plot the signal
    #
    $p.chan0_plot itemconfigure $sig_plot_0 -start_time $window_start \
	    -end_time $window_end -height $signal_ht -width $plot_width \
	    -gain_factor $amplitude_gain -file $audio_file \
	    -num_chan $num_channels -display_chan $disp_chan -borderwidth 0


    # plot the spectrogram of the signal is enable flag is set
    #
    if {$spec_enable == "on"} {

	$p.spec0_plot itemconfigure $spec_plot_0 -start_time $window_start \
		-end_time $window_end -height $spec_ht -width $plot_width \
		-use_preemph $preemph_flag -filter $window_option \
		-framelen $frame_len -preemph_coeff $preemph_coeff \
		-brightness $brightness_d -contrast $contrast_d \
		-sample_freq $sample_frequency -file $audio_file \
		-num_chan $num_channels -display_chan $disp_chan \
		-borderwidth 0 -amp_scale $amplitude_gain -winlen $window_len
    }
	
    # compute the location of the bracket start and end lines
    #
    set x_start [eval expr int(($bracket_start-$window_start) * \
            ($plot_width / $window_duration))]
    set x_end [eval expr int(($bracket_end-$window_start) * \
            ($plot_width / $window_duration))]
    
    # draw a line at the start and end time
    #
    $p.chan0_plot coords $bracket_signal_start_line $x_start 0 \
	    $x_start [winfo height $p.chan0_plot]
    $p.chan0_plot coords $bracket_signal_end_line $x_end 0 $x_end \
	    [winfo height $p.chan0_plot]

    $p.energy0_plot coords $bracket_energy_start_line $x_start 0 \
	    $x_start [winfo height $p.energy0_plot]
    $p.energy0_plot coords $bracket_energy_end_line $x_end 0 $x_end \
	    [winfo height $p.energy0_plot]

    $p.spec0_plot coords $bracket_spec_start_line $x_start 0 \
	    $x_start [winfo height $p.spec0_plot]
    $p.spec0_plot coords $bracket_spec_end_line $x_end 0 $x_end \
	    [winfo height $p.spec0_plot]

    # remove all old time lines
    #
    $p.energy0_plot delete t1_line
    $p.energy0_plot delete energy_line1
    $p.energy0_plot delete energy_line2
    $p.energy0_plot delete energy_scl_mrk
    $p.energy0_plot delete energy_markers
    $p.chan0_plot delete t2_line
    $p.chan0_plot delete chan_line1
    $p.chan0_plot delete chan_line2
    $p.spec0_plot delete t3_line
    $p.spec0_plot delete spec_scl_mrk
    $p.spec0_plot delete spec_markers
    
    # plot the energy plot dB lines with energy length 10 * log((2^15)^2)
    #
    set energy_len 90.3               
    set energy_plot_height [winfo height $p.energy0_plot]
    set energy_sep [expr $energy_plot_height / $energy_len]
    set color3 "grey"

    # plot the signal plot horizontal lines
    #
    set signal_len [winfo height $p.chan0_plot]
    set color2 "grey"

    # plot the spectrogram frequency lines
    #
    set spec_len [expr $sample_frequency / 2000]
    set spec_plot_height [winfo height $p.spec0_plot]
    set spec_sep [expr $spec_plot_height / $spec_len]
    set color1 "white"

    # set the pixel per second
    #
    set pix_per_sec [expr $plot_width / ($window_end - $window_start)]
    set approx_frames [expr $window_start / $tscale_frame]
    set tdiff [expr ceil($approx_frames) - $approx_frames]
    set pixel_offset [expr int(round($tscale_frame * $pix_per_sec * $tdiff))]

    # set the start time position
    #
    set t [expr ceil($window_start / $frame_len) * $frame_len]

    # set the initial tick mark boundaries
    #
    set ultra_diff [expr $t - round($t / $ultra) * $ultra]
    if {$ultra_diff > 0} {
	set ultra_bound [expr $t + ($ultra - $ultra_diff)]
    } else {
	set ultra_bound [expr $t - $ultra_diff]
    }
    set super_diff [expr $t - round($t / $super) * $super]
    if {$super_diff > 0} {
	set super_bound [expr $t + ($super - $super_diff)]
    } else {
	set super_bound [expr $t - $super_diff]
    }
    set major_diff [expr $t - round($t / $major) * $major]
    if {$major_diff > 0} {
	set major_bound [expr $t + ($major - $major_diff)]
    } else {
	set major_bound [expr $t - $major_diff]
    }

    # set the energy plot scale value and magnitude
    #
    set value [expr int($energy_len)]
    set magnitude "dB"

    # draw the energy plot scale labels
    #
    for {set i 0} {$i < $energy_plot_height} \
	    {set i [expr $i + $energy_sep]} {

	# set the scale number
	#
	set text "$value$magnitude"

	if {([expr $value % 20] == 0) && ($i > 0)} {

	    # draw the labels
	    #
	    $p.energy0_plot create text 3 $i -text $text -tag energy_scl_mrk \
		    -anchor w -font $font_3
	}
	
	# decrement the scale value
	#
	set value [expr $value - 1]
    }

    # set the spectrogram scale value and magnitude
    #
    set value [expr int($spec_len)]
    set magnitude "K"

    # draw the spectrogram scale labels
    #
    for {set i 0} {$i < $spec_plot_height} \
	    {set i [expr $i + $spec_sep]} {

	# set the scale number
	#
	set text "$value$magnitude"

	if {$i > 0} {

	    # draw the labels
	    #
	    $p.spec0_plot create text 3 $i -text $text -tag spec_scl_mrk \
		    -fill $color1 -anchor w -font $font_3
	}
	
	# decrement the scale value
	#
	set value [expr $value - 1]
    }

    # set the tick mark deviation
    #
    set deviation [expr $tscale_frame / 1000.0]

    # plot the scale and tick marks
    #
    while {$t < $window_end} {

	# set height of the tick mark
	#
	if {[expr $t + $deviation] >= $ultra_bound} {
	    set flag_ultra 1
	    set flag_super -1
	    set flag_major -1
	    set flag_minor 0
	    set ultra_bound [expr $ultra_bound + $ultra]
	    set super_bound [expr $super_bound + $super]
	    set major_bound [expr $major_bound + $major]	    
	} elseif {[expr $t + $deviation] >= $super_bound} {
	    set flag_ultra -1
	    set flag_super 1
	    set flag_major -1
	    set flag_minor 0
	    set super_bound [expr $super_bound + $super]
	    set major_bound [expr $major_bound + $major]
	} elseif {[expr $t + $deviation] >= $major_bound} {
	    set flag_ultra -1
	    set flag_super -1
	    set flag_major 1
	    set flag_minor 0
	    set major_bound [expr $major_bound + $major]
	} else {
	    set flag_ultra -1
	    set flag_super -1
	    set flag_major -1
	    set flag_minor 1
	}
	
	if {$flag_ultra > 0} {
	    set lh $tscale_ht_ultra
	} elseif {$flag_super > 0} {
	    set lh $tscale_ht_super
	} elseif {$flag_major > 0} {
	    set lh $tscale_ht_major
	} else {	    
	    set lh $tscale_ht_minor
	}

	# set the coordinates
	#
	set xx [eval expr round(($t - $window_start) * \
		($plot_width / $window_duration))]

	# draw the line
	#
	$p.energy0_plot create line $xx 0 $xx $lh -tag t1_line
	$p.energy0_plot create line $xx [winfo height $p.energy0_plot] \
		$xx [expr [winfo height $p.energy0_plot] - $lh] -tag t1_line

	# draw the line
	#
	$p.chan0_plot create line $xx 0 $xx $lh -tag t2_line
	$p.chan0_plot create line $xx [winfo height $p.chan0_plot] \
		$xx [expr [winfo height $p.chan0_plot] - $lh] -tag t2_line

	# draw the line
	#
	$p.spec0_plot create line $xx 0 $xx $lh -tag t3_line
	$p.spec0_plot create line $xx [winfo height $p.spec0_plot] \
		$xx [expr [winfo height $p.spec0_plot] - $lh] -tag t3_line

	# draw the signal plot horizontal lines
	#
	$p.chan0_plot create text $xx [expr $signal_len * 0.75] \
		-text "." -fill $color2 -tag chan_line1
	$p.chan0_plot create text $xx [expr $signal_len * 0.25] \
		-text "." -fill $color2 -tag chan_line2
	
	# set internal loop counter
	#
	set count [expr int($energy_len)]

	# draw the energy plot dB lines
	#
	for {set i 0} {$i < $energy_plot_height} \
		{set i [expr $i + $energy_sep]} {

	    # make sure that the dots don't overwrite the scale markers
	    #
	    if {$xx < 25} {
		break
	    }
	    
	    if {([expr $count % 20] == 0) && ($i > 0)} {

		# draw the line
		#
		$p.energy0_plot create text $xx $i -text "." \
			-fill $color3 -tag energy_markers -font $font_3
	    }

	    # increment the internal counter
	    #
	    set count [expr $count - 1]
	}	        

	# draw the spectrogram horizontal frequency lines
	#
	for {set i 0} {$i < $spec_plot_height} \
		{set i [expr $i + $spec_sep]} {
	    
	    # make sure that the dots don't overwrite the scale markers
	    #
	    if {$xx < 15} {
		break
	    }

	    # draw the line
	    #
	    $p.spec0_plot create text $xx $i -text "." \
		    -fill $color1 -tag spec_markers -font $font_3
	}	        
	
	# increment time stamp
	#
	if {$window_duration < 2} {
	    set t [expr $t + $tscale_frame]
	} else {
	    set t [expr $t + $major]
	}
    }
}

# procedure: plot_turns_proc
#
# arguments:
#  uttlist: utterance list
#  ch: input channel
#  y_in: y coordinate
#
# return: none
#
# procedure to plot the turn display
#
proc plot_turns_proc {uttlist ch y_in} {

    # declare globals
    #
    global window_start
    global window_end
    global window_duration

    global plot_width
    global maximum_time

    global font_0

    global p

    # set the coordinates for the text labels
    #
    set y_t [expr $y_in-5]

    # get the start and end time for each utterance
    #
    foreach item $uttlist {
	foreach {cid uid spkr sex age start end trans} $item {
	    
	    # check if this utterance is covered by the window
	    #
	    set tstart $maximum_time
	    set tend 0
	    if {$start < $window_end} {
		if {$start >= $window_start} {
		    set tstart $start
		} else {
		    set tstart $window_start
		}
	    }
	    
	    if {$end >= $window_start} {
		if {$end <= $window_end} {
		    set tend $end
		} else {
		    set tend $window_end
		}
	    }

	    # draw line only if end is after start
	    #
	    if {$tstart < $tend} {
		set x1 [eval expr ($plot_width / $window_duration) \
			* ($tstart - $window_start)]
		set x2 [eval expr ($plot_width / $window_duration) \
			* ($tend - $window_start)]

		# set the id string
		#
		set uttid [format "%s-%.4d" $ch $uid]
		set x_t [expr $x1+20]

		# plot the line
		#
		$p.trn_plot create line $x1 $y_in $x2 $y_in -width 1.0 \
			-arrow both -arrowshape {1 2 4}
		$p.trn_plot create text $x_t $y_t -text $uttid -font $font_0
	    }
	}
    }
}

# procedure: update_gain_proc
#
# arguments:
#  gain: amplitude gain
#
# return: none
#
# procedure to update the amplitude gain
#
proc update_gain_proc {gain} {

    # declare globals 
    #
    global amplitude_gain
    global prev_amplitude_gain
    
    # update the display only when the value changes
    #
    if {$prev_amplitude_gain == $amplitude_gain} {
	return
    }

    # update the amplitude gain value and display
    #
    set amplitude_gain $gain
    set prev_amplitude_gain $gain

    # update display
    #
    update_window
}

# procedure: update_brightness_proc
#
# arguments:
#  gain: brightness gain
#
# return: none
#
# procedure to update the brightness gain
#
proc update_brightness_proc {gain} {

    # declare globals 
    #
    global brightness_d
    global prev_brightness_d
    
    # update the display only when the value changes
    #
    if {$prev_brightness_d == $brightness_d} {
	return
    }

    # update the amplitude gain value and display
    #
    set brightness_d $gain
    set prev_brightness_d $gain

    # update display
    #
    update_window
}

# procedure: update_contrast_proc
#
# arguments:
#  gain: contrast gain
#
# return: none
#
# procedure to update the contrast gain
#
proc update_contrast_proc {gain} {

    # declare globals 
    #
    global contrast_d
    global prev_contrast_d
    
    # update the display only when the value changes
    #
    if {$prev_contrast_d == $contrast_d} {
	return
    }

    # update the amplitude gain value and display
    #
    set contrast_d $gain
    set prev_contrast_d $gain

    # update display
    #
    update_window
}

# procedure: update_gain_proc
#
# arguments:
#  menu: input pop up menue
#  x: x coordinate
#  y: y coordinate
#  entry {}: input list
# 
# return: none
#
# procedure to make sure that the menu focus problem with Tk 8.0 does
# not cause trouble --- this is basically a rewrite of the original
# Tk function
#
proc tk_popup {menu x y {entry {}}} {
    global tkPriv
    global tcl_platform
    if {($tkPriv(popup) != "") || ($tkPriv(postedMb) != "")} {
        tkMenuUnpost {}
    }
    tkPostOverPoint $menu $x $y $entry
    if {$tcl_platform(platform) == "unix"} {
        tkSaveGrabInfo $menu
        grab -global $menu
        set tkPriv(popup) $menu
        tk_menuSetFocus $menu;
    }
}

#
# end of file
