#! /usr/local/bin/wish -f

# file: $ISIP_DISPLAY_SIGNAL/gui/src/utter.tcl
#
# functions to manipulate utterance structures
#

# list of procedures:
#
#  set_current_proc {index}
#  next_utt_proc {}
#  prev_utt_proc {}
#  find_current_proc{}
#  clear_list_proc{}
#

# procedure: set_current_proc
#
# arguments:
#  index: input utterance index
#
# return: none
#
# procedure to set the current focus
#
proc set_current_proc {index} {

    # declare globals
    #
    global cutt_start
    global cutt_end
    global maximum_time

    global curr_ind
    global current_channel

    global audio_file
    global audiolist

    global conv_id

    global bookmark_flag
    global bm_start

    global num_channels
    global sample_num_bytes
    global sample_frequency

    global sig_plot_0
    global energy_plot_0
    global spec_plot_0

    global window_start
    global window_end
    global window_duration
    global window_maximum
    global bracket_start
    global bracket_end

    global word_mode
    global p

    global cutt_audio
    global putt_audio
    global nutt_audio

    global snr
    global snr_flag
    global window_len
    global frame_len
    global preemph_coeff
    global preemph_flag
    global window_option
    global window_size
    global rms_factor
    global amplitude_gain
    global energy_ht
    global signal_ht
  
    global autoplay


    # set the index variables only if necessary
    #
    if {($index >= 0) && ($index <= [llength $audiolist])} {

	# set the current focus
	#
	set curr_ind $index
	
	# get the current audio file
	#
	set audio_file [lindex $audiolist $curr_ind]

	# check if the audio file exists
	#
	if {[file exists $audio_file] == 0} {
	    warning_proc "The file that you have selected does \
		    not exist please select another file"
	    return
	}
	 
	# set the maximum time for the audio file
	#
	set maximum_time [format "%.6f" [snap_sample_proc [eval expr \
		[file size $audio_file] / ($sample_num_bytes * \
		$sample_frequency * $num_channels)]]]

	# set the maximum window size equal to the maximum time if the 
	# maximum time is less that maximum window size
	#
	if {$maximum_time < $window_maximum} {

	    # set the default start and end times
	    #
	    set cutt_start 0
	    set cutt_end $maximum_time
	    set window_start 0
	    set window_end $maximum_time
	    set window_duration $maximum_time
	    set bracket_start 0
	    set bracket_end $maximum_time

	} else {

	    # set the default start and end times
	    #
	    set cutt_start 0
	    set cutt_end $window_maximum
	    set window_start 0
	    set window_end $window_maximum
	    set window_duration $window_maximum
	    set bracket_start 0
	    set bracket_end $window_maximum
	}

    } elseif {$curr_ind < 0} {
	
	# exit gracefully
	#
	return
    }
    
    # activate the listbox
    #
    $p.list selection clear 0 end
    $p.list see $curr_ind
    $p.list selection set $curr_ind
	
    # set the current window display
    #
    curr_window_proc

    # calculate the SNR of the current signal
    #
    if {$snr_flag == "Off"} {
	set snr 0
    } else {
	set snr [snr_proc]
    }
}

# procedure: next_utt_proc
#
# arguments: none
#
# return: none
#
# procedure to set the focus on next utterance
#
proc next_utt_proc {} {
    
    # declare globals
    #
    global curr_ind
    global audiolist

    global next_flag
    global prev_flag

    global p

    # dont go to the next utterance if busy
    #
    if {$next_flag == 1 || $prev_flag == 1} {
	return
    }

    # set the flag to indicate transition to the next utterance
    #
    set next_flag 1

    # save the current utternace in the uttlist
    #
    bracket_marks_proc

    # sort the utterances
    #
    reorder_proc

    # get the current selected element in the listbox
    #
    set next [expr $curr_ind + 1]

    if {$next >= [llength $audiolist]} {

	# activate the listbox
	#
	$p.list selection clear 0 end
	$p.list see $curr_ind
	$p.list selection set $curr_ind

	# reset the next flag
	#
	set next_flag 0

	return
    }
    
    # set the focus to next
    #
    set_current_proc $next

    # reset the next flag
    #
    set next_flag 0
}

# procedure: prev_utt_proc
#
# arguments: none
#
# return: none
#
# procedure to set the focus on previous utterance
#
proc prev_utt_proc {} {
    
    # declare globals
    #
    global curr_ind    
    global audiolist

    global next_flag
    global prev_flag

    global p

    # dont go to the previous utterance if busy
    #
    if {$next_flag == 1 || $prev_flag == 1} {
	return
    }

    # set the flag to indicate transition to the previous utterance
    #
    set prev_flag 1

    # save the current utternace in the uttlist
    #
    bracket_marks_proc

    # sort the utterances
    #
    reorder_proc

    # get the current selected element in the listbox
    #
    set prev [expr $curr_ind - 1]

    if {$prev < 0} {

	# activate the listbox
	#
	$p.list selection clear 0 end
	$p.list see $prev
	$p.list selection set $prev

	# reset the previous flag
	#
	set prev_flag 0

	return
    }
    
    # set the focus to next
    #
    set_current_proc $prev

    # reset the previous flag
    #
    set prev_flag 0
}

# procedure: clear_list_proc
#
# arguments: none
#
# return: none
#
# procedure to clear selected item in the list box
#
proc clear_list_proc {} {

    # declare globals
    #
    global p
    global audiolist

    global sig_plot_0
    global energy_plot_0
    global spec_plot_0

    global amplitude_gain
    global frame_len

    # get the list of utterances selected to be deleted
    #
    set select_list [$p.list curselection]
    set first [lindex $select_list 0]
    set last [lindex $select_list end]

    # now delete the selected audio files
    #
    set new_audiolist ""

    for {set i 0} {$i < $first} {incr i} {
	lappend new_audiolist [lindex $audiolist $i]
    }

    for {set i [expr $last + 1]} {$i < [llength $audiolist]} {incr i} {
	lappend new_audiolist [lindex $audiolist $i]
    }
    
    # assign the new audio list
    #
    set audiolist $new_audiolist

    # update the list box
    #
    fill_list_proc

    # set the current utterance focus to the new one
    #
    if {[llength $audiolist] == 0} {

	# make the existing plots disappear
	#
	$p.energy0_plot itemconfigure $energy_plot_0 -amp_scale 0
	$p.chan0_plot itemconfigure $sig_plot_0 -gain_factor 0
	$p.spec0_plot itemconfigure $spec_plot_0 -framelen 0

    } elseif {[expr $first - 1] < 0} {
	set_current_proc 0

    } else {
	set_current_proc [expr $first - 1]

    }
}

#
# end of file
