
int32 transfer(file,dt,nsamples,samplesBeforeStart)
     SampFile *file;     /* File I/O class */
     NAport *dt;  /* Descriptor for connection to DAT-Link */
     int32 nsamples; /* Number of samples to transfer or -1 */
     int32 *samplesBeforeStart;
{
  int i;	        
  int32 byteCount = 0;	 /* Number of samples processed */
  int tfdebug = DebugCheck("transfer",2);
  int bitsPerSample = file->precision * ((file->channels==3)?2:1);

  /* Make sure each buffer contains an integer number of samples */
  int32 bufsize = (BUFSIZE*4/bitsPerSample)*(bitsPerSample/4);
  int32 bufFilled = 0;	/* Number of bytes already in \'buffer\' */
  static byte *buffer = 0;
  
  Debug("transfer",1,"sampleSize=%d, bitsPerSample=%d\n",
	file->sampleSize, bitsPerSample);
  
  if (buffer == 0) {
    buffer = (byte *)malloc(BUFSIZE);   /* this is done - issac */
  }
  
  /* Main record loop */
  while ((nsamples < 0) ||
	 (byteCount < SamplesToBytes(nsamples,bitsPerSample))) {
    
    int32 numToWrite,nwrote;
    int32 nread = (*file->read)(file,&buffer[bufFilled],bufsize-bufFilled);
    
    // This would be the place to do the IO manipulation
    //
    if (out_mode == NA_CHANNELS_LEFT) {
      for (i = 0; i < nread; i+=4) {
	buffer[i+2] = buffer[i];
	buffer[i+3] = buffer[i+1];
      }
    }
    else if (out_mode == NA_CHANNELS_RIGHT) {
      for (i = 0; i < nread; i+=4) {
	buffer[i] = buffer[i+2];
	buffer[i+1] = buffer[i+3];	
      }
    }

    if (nread <= 0) {
      return BytesToSamples(byteCount,bitsPerSample);
    }
    if (nsamples >= 0) {
      numToWrite = min(SamplesToBytes(nsamples,bitsPerSample)-byteCount,
		       nread+bufFilled);
    }
    else {
      numToWrite = nread+bufFilled;
    }
    
    assert(numToWrite > 0);
    
    nwrote = NAWrite(dt,(char *)buffer,numToWrite);
    
    if (nwrote > 0) {
      byteCount += nwrote;
    }
    if (nwrote != numToWrite) {
      if ((nwrote >= 0)) {

	NAinfo tmpinfo;
	NAGetInfo(dt,&tmpinfo);
	fprintf(stderr,"Transfer error: %s\n",
		_NAxfrErrorNames[(int)tmpinfo.playback.error]);
	return -BytesToSamples(byteCount,bitsPerSample);
      }
      perror("NAWrite");
      cleanup(7);
    }
	  
    if (tfdebug) {
      Debug("transfer",2,"Transferred %ld bytes; count=%ld, nsamples=%ld\n",
	    (long)nwrote,(long)BytesToSamples(byteCount,bitsPerSample),
	    (long)nsamples);
    }
    
    if (samplesBeforeStart) {
      *samplesBeforeStart -= BytesToSamples(nwrote,bitsPerSample);
      if (*samplesBeforeStart <= 0) {
	Debug("transfer",1,"Starting output\n");
	NABegin(dt,NA_PLAY);
	samplesBeforeStart = 0;
      }
    }
  }
}
