// file: sigpl_bbox_0.cc
//

// isip include files
//
#ifndef __ISIP_SIGNAL_PLOT
#include "signal_plot.h"
#endif

#ifndef __ISIP_SIGNAL_PLOT_CONSTANTS
#include "signal_plot_constants.h"
#endif

// function: signal_plot_bbox_cc
//
// parameters:
//     Tcl_Interp* interpreter: interpreter for this widget
//     Tk_Canvas canvas: canvas containing this widget
//     Signal_plot_item* sig_plot: signal plot widget
//
// returns: int indicating status
//
// this function computes the bounding box of all the pixes that may be
// drawn as part of this item. in addition it recomputes all of the
// geometry information used to display a text item or check for mouse
// hits
//
int signal_plot_bbox_cc(Tcl_Interp* interpreter, Tk_Canvas canvas,
			Signal_plot_item* sig_plot) {

  // declare local variables
  //
  int left_x, top_y;

  // declare buffer memory
  //
  char buffer[SIGPLOT_POLY_VERTICES*2][ISIP_MAX_STRING_LENGTH];
  
  // find the upper left coordinates of the widget round to the nearest
  // pixel
  //
  left_x = (int) (sig_plot->x_loc);
  top_y = (int) (sig_plot->y_loc);

  // set the bounding box for the item - if display is cut off then we may
  // need a fudge factor here
  //
  sig_plot->header.x1 = left_x;
  sig_plot->header.y1 = top_y;
  sig_plot->header.x2 = left_x + sig_plot->width;
  sig_plot->header.y2 = top_y + sig_plot->height;  
  
  // compute the canvas coordinates of a polygon describing the area taken
  // up by this widget. this has to be done by first writing to strings
  // and then getting the coords from tk using those strings. 
  //
  sprintf(buffer[0], "%d", sig_plot->header.x1);
  sprintf(buffer[1], "%d", sig_plot->header.y1);
  sprintf(buffer[2], "%d", sig_plot->header.x2);
  sprintf(buffer[3], "%d", sig_plot->header.y1);
  sprintf(buffer[4], "%d", sig_plot->header.x2);
  sprintf(buffer[5], "%d", sig_plot->header.y2);
  sprintf(buffer[6], "%d", sig_plot->header.x1);
  sprintf(buffer[7], "%d", sig_plot->header.y2);

  // toggle through and convert each vertices
  //
  for (int_4 i = 0; i < 2*SIGPLOT_POLY_VERTICES; i++) {
    if (Tk_CanvasGetCoord(interpreter, canvas, buffer[i],
			  &sig_plot->coord_ptr[i]) != TCL_OK) {
      return TCL_ERROR;
    }
  }

  // exit gracefully
  //
  return TCL_OK;
}










